@@ -423,8 +423,14 @@ chat_window_contact_menu_update (EmpathyChatWindowPriv *priv,
 
 	if (orig_submenu == NULL || !gtk_widget_get_visible (orig_submenu)) {
 		submenu = empathy_chat_get_contact_menu (priv->current_chat);
-		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);
-		gtk_widget_show (menu);
+		if (submenu != NULL) {
+			gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);
+			gtk_widget_show (menu);
+			gtk_widget_set_sensitive (menu, TRUE);
+		}
+		else {
+			gtk_widget_set_sensitive (menu, FALSE);
+		}
 	} else {
 		tp_g_signal_connect_object (orig_submenu,
 					     "notify::visible",
